def pull_exch_list():
    cur_list = [
                "AMEX",
                "AMS",
                "AQS",
                "ASX",
                "ATH",
                "BER",
                "BME",
                "BRU",
                "BSE",
                "BUD",
                "BUE",
                "BVC",
                "CAI",
                "CBOE",
                "CNQ",
                "CPH",
                "DFM",
                "DOH",
                "DUS",
                "DXE",
                "EGX",
                "EURONEXT",
                "HAM",
                "HEL",
                "HKSE",
                "HOSE",
                "ICE",
                "IOB",
                "IST",
                "JKT",
                "JNB",
                "JPX",
                "KLS",
                "KOE",
                "KSC",
                "KUW",
                "LSE",
                "MCX",
                "MEX",
                "MIL",
                "MUN",
                "NASDAQ",
                "NEO",
                "NSE",
                "NYSE",
                "NZE",
                "OEM",
                "OQB",
                "OSL",
                "OTC",
                "PNK",
                "PRA",
                "RIS",
                "SAO",
                "SAU",
                "SES",
                "SET",
                "SGO",
                "SHH",
                "SHZ",
                "SIX",
                "STO",
                "STU",
                "TAI",
                "TLV",
                "TSX",
                "TSXV",
                "TWO",
                "VIE",
                "WSE",
                "XETRA"
                ]

    return cur_list